@@ -5,6 +5,7 @@
 # Run this script to re-generate all maintainer-generated files.
 #
 # Copyright (C) 1999 Martin Grabmueller <mgrabmue@cs.tu-berlin.de>
+# Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
 #
 # This is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +26,14 @@
 here=`pwd`
 cd `dirname $0`
 
+# let automake find this automatic created file
+if ! test -f doc/serveez-api.texi; then
+cat <<EOF > doc/serveez-api.texi
+@setfilename serveez-api.info
+EOF
+info_touched="yes"
+fi
+
 echo -n "Creating aclocal.m4... "
 aclocal
 echo "done."
@@ -38,6 +47,9 @@ echo -n "Creating configure... "
 autoconf
 echo "done."
 
+# reschedule this file for building
+if test x"$info_touched" = xyes ; then rm -f doc/serveez-api.texi; fi
+
 #
 # run configure, maybe with parameters recorded in config.status
 #
