@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: control-proto.c,v 1.54 2001/06/27 20:38:36 ela Exp $
+ * $Id: control-proto.c,v 1.57 2001/08/12 10:59:04 ela Exp $
  *
  */
 
@@ -348,11 +348,12 @@ ctrl_stat_id (svz_socket_t *sock, int flag, char *arg)
       /* usual client */
       if ((server = svz_server_find (xsock->cfg)) != NULL)
 	{
+	  char *info;
 	  svz_sock_printf (sock, "%s client\r\n", server->name);
-	  if (server->info_client)
+	  if (server->info_client && 
+	      (info = server->info_client (server, xsock)) != NULL)
 	    {
-	      svz_sock_printf (sock, "            %s\r\n",
-			       server->info_client (server, xsock));
+	      svz_sock_printf (sock, "            %s\r\n", info);
 	    }
 	}
       /* coserver */
@@ -370,42 +371,52 @@ ctrl_stat_id (svz_socket_t *sock, int flag, char *arg)
     }
 
   /* print all previously collected statistics of this connection */
-  svz_sock_printf (sock, 
-		   " sock fd  : %d\r\n"
-		   " file fd  : %d\r\n"
-		   " pipe fd  : %d (recv), %d (send)\r\n"
-		   " foreign  : %s:%u\r\n",
-		   xsock->sock_desc,
-		   xsock->file_desc,
-		   xsock->pipe_desc[READ],
-		   xsock->pipe_desc[WRITE],
-		   svz_inet_ntoa (xsock->remote_addr),
-		   ntohs (xsock->remote_port));
+  if (xsock->flags & SOCK_FLAG_SOCK)
+    svz_sock_printf (sock, " sock fd  : %d\r\n", xsock->sock_desc);
+  if (xsock->flags & SOCK_FLAG_FILE)
+    svz_sock_printf (sock, " file fd  : %d\r\n", xsock->file_desc);
+  if (xsock->flags & SOCK_FLAG_PIPE)
+    svz_sock_printf (sock, " pipe fd  : %d (recv), %d (send)\r\n",
+		     xsock->pipe_desc[READ], 
+		     xsock->pipe_desc[WRITE]);
+
+  if (xsock->flags & SOCK_FLAG_PIPE)
+    {
+      if (xsock->send_pipe)
+	svz_sock_printf (sock, " foreign  : %s\r\n", xsock->send_pipe);
+      if (xsock->recv_pipe)
+	svz_sock_printf (sock, " local    : %s\r\n", xsock->recv_pipe);
+    }
+  if (xsock->flags & SOCK_FLAG_SOCK)
+    {
+      svz_sock_printf (sock, " foreign  : %s:%u\r\n",
+		       svz_inet_ntoa (xsock->remote_addr),
+		       ntohs (xsock->remote_port));
+      svz_sock_printf (sock, " local    : %s:%u\r\n",
+		       svz_inet_ntoa (xsock->local_addr),
+		       ntohs (xsock->local_port));
+    }
 
-  /* that is why `svz_inet_ntoa' cannot be used in the same call */
   svz_sock_printf (sock, 
-	       " local    : %s:%u\r\n"
-	       " sendbuf  : %d (size), %d (fill), %s (last send)\r\n"
-	       " recvbuf  : %d (size), %d (fill), %s (last recv)\r\n"
-	       " idle     : %d\r\n"
+		   " sendbuf  : %d (size), %d (fill), %s (last send)\r\n"
+		   " recvbuf  : %d (size), %d (fill), %s (last recv)\r\n"
+		   " idle     : %d\r\n"
 #if ENABLE_FLOOD_PROTECTION
-	       " flood    : %d (points), %d (limit)\r\n"
+		   " flood    : %d (points), %d (limit)\r\n"
 #endif /* ENABLE_FLOOD_PROTECTION */
-	       " avail    : %s\r\n\r\n",
-	       svz_inet_ntoa (xsock->local_addr),
-	       ntohs (xsock->local_port),
-	       xsock->send_buffer_size,
-	       xsock->send_buffer_fill,
-	       svz_time (xsock->last_send),
-	       xsock->recv_buffer_size,
-	       xsock->recv_buffer_fill,
-	       svz_time (xsock->last_recv),
-	       xsock->idle_counter,
+		   " avail    : %s\r\n\r\n",
+		   xsock->send_buffer_size,
+		   xsock->send_buffer_fill,
+		   svz_time (xsock->last_send),
+		   xsock->recv_buffer_size,
+		   xsock->recv_buffer_fill,
+		   svz_time (xsock->last_recv),
+		   xsock->idle_counter,
 #if ENABLE_FLOOD_PROTECTION
-	       xsock->flood_points,
-	       xsock->flood_limit,
+		   xsock->flood_points,
+		   xsock->flood_limit,
 #endif /* ENABLE_FLOOD_PROTECTION */
-	       xsock->unavailable ? "no" : "yes");
+		   xsock->unavailable ? "no" : "yes");
 
   return flag;
 }
@@ -432,9 +443,9 @@ ctrl_stat (svz_socket_t *sock, int flag, char *arg)
     {
       svz_sock_printf (sock, "\r\n%s (%s):\r\n",
 		       server->description, server->name);
-      if (server->info_server)
+      if (server->info_server && (p = server->info_server (server)) != NULL)
 	{
-	  svz_sock_printf (sock, "%s\r\n", server->info_server (server));
+	  svz_sock_printf (sock, "%s\r\n", p);
 	}
       svz_sock_printf (sock, "\r\n");
       return flag;
@@ -568,32 +579,25 @@ ctrl_stat_cache (svz_socket_t *sock, int flag, char *arg)
 {
   int n, total, files;
   char *p;
-  http_cache_entry_t **cache;
+  http_cache_entry_t *cache;
 
   svz_sock_printf (sock, "\r\n%s", 
 		   "File                             "
 		   "Size  Usage  Hits Recent Ready\r\n");
 
-  files = total = 0;
-  if ((cache = (http_cache_entry_t **) svz_hash_values (http_cache)) != NULL)
+  files = total = n = 0;
+  /* go through each cache entry */
+  for (cache = http_cache_first; cache; cache = cache->next, n++)
     {
-      /* go through each cache entry */
-      for (n = 0; n < svz_hash_size (http_cache); n++)
-	{
-	  files++;
-	  total += cache[n]->size;
-	  p = cache[n]->file;
-	  p += strlen (cache[n]->file);
-	  while (*p != '/' && *p != '\\' && p != cache[n]->file) p--;
-	  if (p != cache[n]->file) p++;
-	  svz_sock_printf (sock, "%-30s %6d %6d %5d %6d %-5s\r\n", p,
-			   cache[n]->size,
-			   cache[n]->usage,
-			   cache[n]->hits,
-			   cache[n]->urgent,
-			   cache[n]->ready ? "Yes" : "No");
-	}
-      svz_hash_xfree (cache);
+      files++;
+      total += cache->size;
+      p = cache->file;
+      p += strlen (cache->file);
+      while (*p != '/' && *p != '\\' && p != cache->file) p--;
+      if (p != cache->file) p++;
+      svz_sock_printf (sock, "%-30s %6d %6d %5d %6d %-5s\r\n", p,
+		       cache->size, cache->usage, cache->hits, n,
+		       cache->ready ? "Yes" : "No");
     }
 
   /* print cache summary */
@@ -993,7 +997,7 @@ ctrl_get_cpu_state (void)
 int
 ctrl_idle (svz_socket_t *sock)
 {
-  int n, old;
+  int n, old, ret;
   unsigned long all;
   cpu_state_t *c = &cpu_state;
 
@@ -1001,23 +1005,25 @@ ctrl_idle (svz_socket_t *sock)
   n = (c->index + 1) & 1;
 
   /* get status of the cpu and process */
-  if (ctrl_get_cpu_state () != -1)
-    {
-      /* calculate process specific info */
-      all = c->ptotal[n] - c->ptotal[old]; 
-      if (all != 0)
-	{
-	  svz_snprintf (c->pinfo, STAT_BUFFER_SIZE, PROC_FORMAT,
-			PROC_DIFF (0) * 100 / all,
-			PROC_DIFF (0) * 1000 / all % 10,
-			PROC_DIFF (1) * 100 / all,
-			PROC_DIFF (1) * 1000 / all % 10,
-			PROC_DIFF (2) * 100 / all,
-			PROC_DIFF (2) * 1000 / all % 10,
-			PROC_DIFF (3) * 100 / all,
-			PROC_DIFF (3) * 1000 / all % 10);
-	}
+  ret = ctrl_get_cpu_state ();
 
+  /* calculate process specific info */
+  all = c->ptotal[n] - c->ptotal[old]; 
+  if (all != 0)
+    {
+      svz_snprintf (c->pinfo, STAT_BUFFER_SIZE, PROC_FORMAT,
+		    PROC_DIFF (0) * 100 / all,
+		    PROC_DIFF (0) * 1000 / all % 10,
+		    PROC_DIFF (1) * 100 / all,
+		    PROC_DIFF (1) * 1000 / all % 10,
+		    PROC_DIFF (2) * 100 / all,
+		    PROC_DIFF (2) * 1000 / all % 10,
+		    PROC_DIFF (3) * 100 / all,
+		    PROC_DIFF (3) * 1000 / all % 10);
+    }
+
+  if (ret != -1)
+    {
       /* calculate cpu specific info */
       c->total[n] = c->cpu[n][0] + c->cpu[n][1] + c->cpu[n][2] + c->cpu[n][3];
       all = c->total[n] - c->total[old];
