@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile.c,v 1.37 2001/07/12 20:23:52 ela Exp $
+ * $Id: guile.c,v 1.44 2001/09/21 13:12:41 ela Exp $
  *
  */
 
@@ -111,7 +111,7 @@ optionhash_destroy (svz_hash_t *options)
 }
 
 /*
- * Save the current load port for later usage in the @code{report_error()}
+ * Save the current load port for later usage in the @code{guile_error()}
  * function.
  */
 static void
@@ -131,7 +131,7 @@ static SCM
 guile_get_current_load_port (void)
 {
   SCM p = scm_current_load_port ();
-  if (SCM_PORTP (p))
+  if (!gh_eq_p (p, SCM_BOOL_F) && SCM_PORTP (p))
     return p;
   if (SCM_PORTP (guile_load_port))
     return guile_load_port;
@@ -144,12 +144,13 @@ guile_get_current_load_port (void)
  * need a trailing newline.
  */
 void
-report_error (const char *format, ...)
+guile_error (char *format, ...)
 {
   va_list args;
   /* FIXME: Why is this port undefined in guile exceptions ? */
   SCM lp = guile_get_current_load_port ();
-  char *file = SCM_PORTP (lp) ? gh_scm2newstr (SCM_FILENAME (lp), NULL) : NULL;
+  char *file = 
+    SCM_PORTP (lp) ? gh_scm2newstr (SCM_FILENAME (lp), NULL) : NULL;
 
   /* guile counts lines from 0, we have to add one */
   fprintf (stderr, "%s:%d:%d: ", file ? file : "undefined", 
@@ -192,8 +193,8 @@ optionhash_validate (svz_hash_t *hash, int what, char *type, char *name)
 	  if (value->defined != 1)
 	    {
 	      errors++;
-	      report_error ("Multiple definitions of `%s' in %s `%s'",
-			    key, type, name);
+	      guile_error ("Multiple definitions of `%s' in %s `%s'",
+			   key, type, name);
 	    }
 	  break;
 	  /* Check usage counter. */
@@ -201,8 +202,7 @@ optionhash_validate (svz_hash_t *hash, int what, char *type, char *name)
 	  if (value->use == 0)
 	    {
 	      errors++;
-	      report_error ("Unused variable `%s' in %s `%s'",
-			    key, type, name);
+	      guile_error ("Unused variable `%s' in %s `%s'", key, type, name);
 	    }
 	  break;
 	}
@@ -238,7 +238,7 @@ optionhash_get (svz_hash_t *hash, char *key)
  * Ela and Raimi...
  */
 svz_hash_t *
-guile2optionhash (SCM pairlist, char *txt, int dounpack)
+guile_to_optionhash (SCM pairlist, char *txt, int dounpack)
 {
   svz_hash_t *hash = svz_hash_create (10);
   guile_value_t *old_value;
@@ -246,7 +246,7 @@ guile2optionhash (SCM pairlist, char *txt, int dounpack)
   int err = 0;
 
   /* Unpack if requested, ignore if null already (null == not existent). */
-  if (dounpack && !gh_null_p (pairlist) && pairlist != SCM_UNDEFINED)
+  if (dounpack && !gh_null_p (pairlist) && !gh_eq_p (pairlist, SCM_UNDEFINED))
     pairlist = gh_car (pairlist);
 
   for ( ; gh_pair_p (pairlist); pairlist = gh_cdr (pairlist))
@@ -258,17 +258,17 @@ guile2optionhash (SCM pairlist, char *txt, int dounpack)
       /* The car must be another pair which contains key and value. */
       if (!gh_pair_p (pair))
 	{
-	  report_error ("Not a pair %s", txt);
+	  guile_error ("Not a pair %s", txt);
 	  err = 1;
 	  break;
 	}
       key = gh_car (pair);
       val = gh_cdr (pair);
 
-      if (NULL == (str = guile2str (key)))
+      if (NULL == (str = guile_to_string (key)))
 	{
 	  /* Unknown key type, must be string or symbol. */
-	  report_error ("Key must be string or symbol %s", txt);
+	  guile_error ("Key must be string or symbol %s", txt);
 	  err = 1;
 	  break;
 	}
@@ -288,7 +288,7 @@ guile2optionhash (SCM pairlist, char *txt, int dounpack)
   /* Pairlist must be gh_null_p() now or that was not a good pairlist. */
   if (!err && !gh_null_p (pairlist))
     {
-      report_error ("Invalid pairlist %s", txt);
+      guile_error ("Invalid pairlist %s", txt);
       err = 1;
     }
 
@@ -307,18 +307,18 @@ guile2optionhash (SCM pairlist, char *txt, int dounpack)
  * error messages.
  */
 int
-guile2int (SCM scm, int *target)
+guile_to_integer (SCM cell, int *target)
 {
   int err = 0;
   char *str = NULL, *endp;
 
-  /* Usual guile number. */
-  if (gh_number_p (scm))
+  /* Usual guile exact number. */
+  if (gh_exact_p (cell))
     {
-      *target = gh_scm2int (scm);
+      *target = gh_scm2int (cell);
     }
   /* Try string (or even symbol) to integer conversion. */
-  else if (NULL != (str = guile2str (scm)))
+  else if (NULL != (str = guile_to_string (cell)))
     {
       errno = 0;
       *target = strtol (str, &endp, 10);
@@ -340,26 +340,26 @@ guile2int (SCM scm, int *target)
  * zero when successful. Stores the boolean/integer where @var{target} points
  * to. Does not emit error messages.
  */
-static int
-guile2boolean (SCM scm, int *target)
+int
+guile_to_boolean (SCM cell, int *target)
 {
   int i;
   int err = 0;
   char *str;
 
   /* Usual guile boolean. */
-  if (gh_boolean_p (scm))
+  if (gh_boolean_p (cell))
     {
-      i = gh_scm2bool (scm);
+      i = gh_scm2bool (cell);
       *target = (i == 0 ? 0 : 1);
     }
   /* Try with the integer converter. */
-  else if (guile2int (scm, &i) == 0)
+  else if (guile_to_integer (cell, &i) == 0)
     {
       *target = (i == 0 ? 0 : 1);
     }
   /* Neither integer nor boolean, try text conversion. */
-  else if ((str = guile2str (scm)) != NULL)
+  else if ((str = guile_to_string (cell)) != NULL)
     {
       if (!svz_strcasecmp (str, "yes") ||
 	  !svz_strcasecmp (str, "on") ||
@@ -379,6 +379,89 @@ guile2boolean (SCM scm, int *target)
   return err;
 }
 
+/*
+ * Convert the given guile list @var{list} into a hash. Return NULL on
+ * failure. Error messages will be emitted if necessary.
+ */
+svz_hash_t *
+guile_to_hash (SCM list, char *prefix)
+{
+  int err = 0, i;
+  svz_hash_t *hash;
+
+  /* Is this valid guile list at all ? */
+  if (!gh_list_p (list))
+    {
+      err = -1;
+      guile_error ("%s: Not a valid list for hash (alist)", prefix);
+      return NULL;
+    }
+
+  /* Iterate the alist. */
+  hash = svz_hash_create (gh_length (list));
+  for (i = 0; gh_pair_p (list); list = gh_cdr (list), i++)
+    {
+      SCM k, v, pair = gh_car (list);
+      char *str, *keystr, *valstr;
+
+      if (!gh_pair_p (pair))
+	{
+	  err = -1;
+	  guile_error ("%s: Element #%d of hash (alist) is not a pair", 
+		       prefix, i);
+	  continue;
+	}
+      k = gh_car (pair);
+      v = gh_cdr (pair);
+	     
+      /* Obtain key character string. */
+      if (NULL == (str = guile_to_string (k)))
+	{
+	  err = -1;
+	  guile_error ("%s: Element #%d of hash (alist) "
+		       "has no valid key (string required)", prefix, i);
+	  keystr = NULL;
+	}
+      else
+	{
+	  keystr = svz_strdup (str);
+	  scm_must_free (str);
+	}
+
+      /* Obtain value character string. */
+      if (NULL == (str = guile_to_string (v)))
+	{
+	  err = -1;
+	  guile_error ("%s: Element #%d of hash (alist) "
+		       "has no valid value (string required)", prefix, i);
+	  valstr = NULL;
+	}
+      else
+	{
+	  valstr = svz_strdup (str);
+	  scm_must_free (str);
+	}
+
+      /* Add to hash if key and value look good. */
+      if (keystr != NULL && valstr != NULL)
+	{
+	  svz_hash_put (hash, keystr, valstr);
+	  svz_free (keystr);
+	}
+    }
+
+  /* Free the values, keys are freed be hash destructor. */
+  if (err)
+    {
+      char **values;
+      svz_hash_foreach_value (hash, values, i)
+	svz_free (values[i]);
+      svz_hash_destroy (hash);
+      return NULL;
+    }
+  return hash;
+}
+
 /*
  * Convert the given scheme cell @var{list} which needs to be a valid guile
  * list into an array of duplicated strings. Returns @code{NULL} if it is not
@@ -386,8 +469,8 @@ guile2boolean (SCM scm, int *target)
  * is not a string. The additional argument @var{func} should be the name of 
  * the calling function.
  */
-static svz_array_t *
-guile2strarray (SCM list, char *func)
+svz_array_t *
+guile_to_strarray (SCM list, char *func)
 {
   svz_array_t *array;
   int i;
@@ -395,15 +478,18 @@ guile2strarray (SCM list, char *func)
 
   /* Check if the given scheme cell is a valid list. */
   if (!gh_list_p (list))
-    return NULL;
+    {
+      guile_error ("%s: String array is not a valid list", func);
+      return NULL;
+    }
 
   /* Iterate over the list and build up the array of strings. */
   array = svz_array_create (gh_length (list));
   for (i = 0; gh_pair_p (list); list = gh_cdr (list), i++)
     {
-      if ((str = guile2str (gh_car (list))) == NULL)
+      if ((str = guile_to_string (gh_car (list))) == NULL)
 	{
-	  report_error ("%s: String expected in position %d", func, i);
+	  guile_error ("%s: String expected in position %d", func, i);
 	  guile_global_error = -1;
 	  continue;
 	}
@@ -420,6 +506,46 @@ guile2strarray (SCM list, char *func)
   return array;
 }
 
+/*
+ * Convert the given scheme cell @var{list} which needs to be a valid guile
+ * list into an array of integers. The additional argument @var{func} is the
+ * name of the calling function. Return NULL on failure.
+ */
+svz_array_t *
+guile_to_intarray (SCM list, char *func)
+{
+  svz_array_t *array;
+  int i, n;
+
+  /* Check if the given scheme cell is a valid associative list. */
+  if (!gh_list_p (list))
+    {
+      guile_error ("%s: Integer array is not a valid list", func);
+      return NULL;
+    }
+
+  /* Iterate over the list and build up the array of strings. */
+  array = svz_array_create (gh_length (list));
+  for (i = 0; gh_pair_p (list); list = gh_cdr (list), i++)
+    {
+      if (guile_to_integer (gh_car (list), &n) != 0)
+	{
+	  guile_error ("%s: Integer expected in position %d", func, i);
+	  guile_global_error = -1;
+	  continue;
+	}
+      svz_array_add (array, SVZ_NUM2PTR (n));
+    }
+
+  /* Check the size of the resulting integer array. */
+  if (svz_array_size (array) == 0)
+    {
+      svz_array_destroy (array);
+      array = NULL;
+    }
+  return array;
+}
+
 /*
  * Extract an integer value from an option hash. Returns zero if it worked.
  */
@@ -434,22 +560,22 @@ optionhash_extract_int (svz_hash_t *hash,
   int err = 0;
   SCM hvalue = optionhash_get (hash, key);
 
-  /* Is there such an interger in the option-hash ? */
-  if (SCM_UNSPECIFIED == hvalue)
+  /* Is there such an integer in the option-hash ? */
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       /* Nothing in hash, try to use default. */
       if (hasdef)
 	*target = defvar;
       else
 	{
-	  report_error ("No default value for integer `%s' %s", key, txt);
+	  guile_error ("No default value for integer `%s' %s", key, txt);
 	  err = 1;
 	}
     }
   /* Convert the integer value. */
-  else if (guile2int (hvalue, target))
+  else if (guile_to_integer (hvalue, target))
     {
-      report_error ("Invalid integer value for `%s' %s", key, txt);
+      guile_error ("Invalid integer value for `%s' %s", key, txt);
       err = 1;
     }
   return err;
@@ -473,23 +599,23 @@ optionhash_extract_string (svz_hash_t *hash,
   char *str = NULL;
 
   /* Is there such a string in the option-hash ? */
-  if (SCM_UNSPECIFIED == hvalue)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       /* Nothing in hash, try to use default. */
       if (hasdef)
 	*target = svz_strdup (defvar);
       else
 	{
-	  report_error ("No default value for string `%s' %s", key, txt);
+	  guile_error ("No default value for string `%s' %s", key, txt);
 	  err = 1;
 	}
     }
   else
     {
       /* Try getting the character string. */
-      if (NULL == (str = guile2str (hvalue)))
+      if (NULL == (str = guile_to_string (hvalue)))
 	{
-	  report_error ("Invalid string value for `%s' %s", key, txt);
+	  guile_error ("Invalid string value for `%s' %s", key, txt);
 	  err = 1;
 	}
       else
@@ -519,19 +645,19 @@ optionhash_cb_integer (char *server, void *arg, char *key, int *target,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
   
-  if (SCM_UNSPECIFIED == hvalue)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
 
-      report_error ("%s: You have to define an integer called `%s'",
-		    server, key);
+      guile_error ("%s: You have to define an integer called `%s'",
+		   server, key);
       return SVZ_ITEM_FAILED;
     }
 
-  if (guile2int (hvalue, target))
+  if (guile_to_integer (hvalue, target))
     {
-      report_error ("%s: Invalid integer value for `%s'", server, key);
+      guile_error ("%s: Invalid integer value for `%s'", server, key);
       return SVZ_ITEM_FAILED;
     }
 
@@ -546,19 +672,19 @@ optionhash_cb_boolean (char *server, void *arg, char *key, int *target,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
 
-  if (SCM_UNSPECIFIED == hvalue)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
 
-      report_error ("%s: You have to define a boolean called `%s'",
-		    server, key);
+      guile_error ("%s: You have to define a boolean called `%s'",
+		   server, key);
       return SVZ_ITEM_FAILED;
     }
 
-  if (guile2boolean (hvalue, target))
+  if (guile_to_boolean (hvalue, target))
     {
-      report_error ("%s: Invalid boolean value for `%s'", server, key);
+      guile_error ("%s: Invalid boolean value for `%s'", server, key);
       return SVZ_ITEM_FAILED;
     }
 
@@ -575,13 +701,13 @@ optionhash_cb_intarray (char *server, void *arg, char *key,
   SCM hvalue = optionhash_get (options, key);
 
   /* Is that integer array defined in the option-hash ? */
-  if (SCM_UNSPECIFIED == hvalue)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
 
-      report_error ("%s: You have to define a integer array called `%s'",
-		    server, key);
+      guile_error ("%s: You have to define a integer array called `%s'",
+		   server, key);
       return SVZ_ITEM_FAILED;
     }
   /* Yes, start parsing it. */
@@ -593,8 +719,8 @@ optionhash_cb_intarray (char *server, void *arg, char *key,
       /* Is it a valid list ? */
       if (!gh_list_p (hvalue))
 	{
-	  report_error ("%s: Integer array `%s' is not a valid list",
-			server, key);
+	  guile_error ("%s: Integer array `%s' is not a valid list",
+		       server, key);
 	  return SVZ_ITEM_FAILED;
 	}
 
@@ -602,14 +728,14 @@ optionhash_cb_intarray (char *server, void *arg, char *key,
       array = svz_array_create (0);
       for (i = 0; gh_pair_p (hvalue); hvalue = gh_cdr (hvalue), i++)
 	{
-	  if (guile2int (gh_car (hvalue), &val))
+	  if (guile_to_integer (gh_car (hvalue), &val))
 	    {
-	      report_error ("%s: Invalid integer element #%d in "
-			    "integer array `%s'", server, i, key);
+	      guile_error ("%s: Invalid integer element #%d in "
+			   "integer array `%s'", server, i, key);
 	      err = -1;
 	      continue;
 	    }
-	  svz_array_add (array, (void *) ((long) val));
+	  svz_array_add (array, SVZ_NUM2PTR (val));
 	}
 
       if (err)
@@ -634,19 +760,19 @@ optionhash_cb_string (char *server, void *arg, char *key,
   SCM hvalue = optionhash_get (options, key);
   char *str;
 
-  if (SCM_UNSPECIFIED == hvalue)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
 
-      report_error ("%s: You have to define a string called `%s'",
-		    server, key);
+      guile_error ("%s: You have to define a string called `%s'",
+		   server, key);
       return SVZ_ITEM_FAILED;
     }
 
-  if (NULL == (str = guile2str (hvalue)))
+  if (NULL == (str = guile_to_string (hvalue)))
     {
-      report_error ("%s: Invalid string value for `%s'", server, key);
+      guile_error ("%s: Invalid string value for `%s'", server, key);
       return SVZ_ITEM_FAILED;
     }
 
@@ -664,13 +790,13 @@ optionhash_cb_strarray (char *server, void *arg, char *key,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
 
-  if (SCM_UNSPECIFIED == hvalue)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
 
-      report_error ("%s: You have to define a string array called `%s'",
-		    server, key);
+      guile_error ("%s: You have to define a string array called `%s'",
+		   server, key);
       return SVZ_ITEM_FAILED;
     }
   else
@@ -682,8 +808,8 @@ optionhash_cb_strarray (char *server, void *arg, char *key,
       /* Check if this is really a list. */
       if (!gh_list_p (hvalue))
 	{
-	  report_error ("%s: String array `%s' is not a valid list",
-			server, key);
+	  guile_error ("%s: String array `%s' is not a valid list",
+		       server, key);
 	  return SVZ_ITEM_FAILED;
 	}
 
@@ -691,10 +817,10 @@ optionhash_cb_strarray (char *server, void *arg, char *key,
       array = svz_array_create (0);
       for (i = 0; gh_pair_p (hvalue); hvalue = gh_cdr (hvalue), i++)
 	{
-	  if (NULL == (str = guile2str (gh_car (hvalue))))
+	  if (NULL == (str = guile_to_string (gh_car (hvalue))))
 	    {
-	      report_error ("%s: Invalid string element #%d in "
-			    "string array `%s'", server, i, key);
+	      guile_error ("%s: Invalid string element #%d in "
+			   "string array `%s'", server, i, key);
 	      err = -1;
 	      continue;
 	    }
@@ -726,13 +852,13 @@ optionhash_cb_hash (char *server, void *arg, char *key,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
 
-  if (SCM_UNSPECIFIED == hvalue)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
 
-      report_error ("%s: You have to define a hash (alist) called `%s'",
-		    server, key);
+      guile_error ("%s: You have to define a hash (alist) called `%s'",
+		   server, key);
       return SVZ_ITEM_FAILED;
     }
   else
@@ -744,8 +870,8 @@ optionhash_cb_hash (char *server, void *arg, char *key,
       if (!gh_list_p (hvalue))
 	{
 	  err = -1;
-	  report_error ("%s: Not a valid list for hash (alist) `%s'",
-			server, key);
+	  guile_error ("%s: Not a valid list for hash (alist) `%s'",
+		       server, key);
 	  return SVZ_ITEM_FAILED;
 	}
 
@@ -759,20 +885,20 @@ optionhash_cb_hash (char *server, void *arg, char *key,
 	  if (!gh_pair_p (pair))
 	    {
 	      err = -1;
-	      report_error ("%s: Element #%d of hash (alist) `%s' "
-			    "is not a pair", server, i, key);
+	      guile_error ("%s: Element #%d of hash (alist) `%s' "
+			   "is not a pair", server, i, key);
 	      continue;
 	    }
 	  k = gh_car (pair);
 	  v = gh_cdr (pair);
 	     
 	  /* Obtain key character string. */
-	  if (NULL == (str = guile2str (k)))
+	  if (NULL == (str = guile_to_string (k)))
 	    {
 	      err = -1;
-	      report_error ("%s: Element #%d of hash (alist) `%s' "
-			    "has no valid key (string required)",
-			    server, i, key);
+	      guile_error ("%s: Element #%d of hash (alist) `%s' "
+			   "has no valid key (string required)",
+			   server, i, key);
 	      keystr = NULL;
 	    }
 	  else
@@ -782,12 +908,12 @@ optionhash_cb_hash (char *server, void *arg, char *key,
 	    }
 
 	  /* Obtain value character string. */
-	  if (NULL == (str = guile2str (v)))
+	  if (NULL == (str = guile_to_string (v)))
 	    {
 	      err = -1;
-	      report_error ("%s: Element #%d of hash (alist) `%s' "
-			    "has no valid value (string required)",
-			    server, i, key);
+	      guile_error ("%s: Element #%d of hash (alist) `%s' "
+			   "has no valid value (string required)",
+			   server, i, key);
 	      valstr = NULL;
 	    }
 	  else
@@ -832,27 +958,27 @@ optionhash_cb_portcfg (char *server, void *arg, char *key,
   char *str;
 
   /* Is the requested port configuration defined ? */
-  if (hvalue == SCM_UNSPECIFIED)
+  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
 
-      report_error ("%s: Port configuration `%s' required", server, key);
+      guile_error ("%s: Port configuration `%s' required", server, key);
       return SVZ_ITEM_FAILED;
     }
 
   /* Convert Scheme cell into string. */
-  if ((str = guile2str (hvalue)) == NULL)
+  if ((str = guile_to_string (hvalue)) == NULL)
     {
-      report_error ("%s: Invalid string or symbol value for `%s'",
-		    server, key);
+      guile_error ("%s: Invalid string or symbol value for `%s'",
+		   server, key);
       return SVZ_ITEM_FAILED;
     }
 
   /* Check existence of this named port configuration . */
   if ((port = svz_portcfg_get (str)) == NULL)
     {
-      report_error ("%s: No such port configuration: `%s'", server, str);
+      guile_error ("%s: No such port configuration: `%s'", server, str);
       scm_must_free (str);
       return SVZ_ITEM_FAILED;
     }
@@ -937,9 +1063,9 @@ guile_define_server (SCM name, SCM args)
   GUILE_PRECALL ();
 
   /* Check if the given server name is valid. */
-  if (NULL == (servername = guile2str (name)))
+  if (NULL == (servername = guile_to_string (name)))
     {
-      report_error ("Invalid server name");
+      guile_error ("Invalid server name");
       FAIL ();
     }
 
@@ -947,9 +1073,9 @@ guile_define_server (SCM name, SCM args)
   svz_snprintf (txt, 256, "defining server `%s'", servername);
 
   /* Extract options if any. */
-  if (args == SCM_UNDEFINED)
+  if (gh_eq_p (args, SCM_UNDEFINED))
     options = svz_hash_create (4);
-  else if (NULL == (options = guile2optionhash (args, txt, 0)))
+  else if (NULL == (options = guile_to_optionhash (args, txt, 0)))
     FAIL (); /* Message already emitted. */
 
   /* Seperate server description. */
@@ -958,18 +1084,18 @@ guile_define_server (SCM name, SCM args)
     p++;
 
   /* Extract server type and sanity check. */
-  if (*p == '-' && *(p+1) != '\0')
+  if (*p == '-' && *(p + 1) != '\0')
     *p = '\0';
   else
     {
-      report_error ("Not a valid server name: `%s'");
+      guile_error ("Not a valid server name: `%s'");
       FAIL ();
     }
 
   /* Find the definition by lookup with dynamic loading. */
   if (NULL == (stype = svz_servertype_get (servertype, 1)))
     {
-      report_error ("No such server type: `%s'", servertype);
+      guile_error ("No such server type: `%s'", servertype);
       FAIL ();
     }
 
@@ -990,7 +1116,7 @@ guile_define_server (SCM name, SCM args)
     {
       if ((server = svz_server_add (server)) != NULL)
 	{
-	  report_error ("Duplicate server definition: `%s'", servername);
+	  guile_error ("Duplicate server definition: `%s'", servername);
 	  svz_server_free (server);
 	  FAIL ();
 	}
@@ -1030,17 +1156,17 @@ guile_define_port (SCM name, SCM args)
   GUILE_PRECALL ();
 
   /* Check validity of first argument. */
-  if ((portname  = guile2str (name)) == NULL)
+  if ((portname  = guile_to_string (name)) == NULL)
     {
-      report_error ("First argument to `" FUNC_NAME "' "
-		    "needs to be string or symbol");
+      guile_error ("First argument to `" FUNC_NAME "' "
+		   "needs to be string or symbol");
       FAIL ();
     }
 
   txt = svz_malloc (256);
   svz_snprintf (txt, 256, "defining port `%s'", portname);
 
-  if (NULL == (options = guile2optionhash (args, txt, 0)))
+  if (NULL == (options = guile_to_optionhash (args, txt, 0)))
     FAIL (); /* Message already emitted. */
 
   /* Every key defined only once ? */
@@ -1048,10 +1174,11 @@ guile_define_port (SCM name, SCM args)
     err = -1;
 
   /* Find out what protocol this portcfg will be about. */
-  if (NULL == (proto = guile2str (optionhash_get (options, PORTCFG_PROTO))))
+  if (NULL == (proto = guile_to_string (optionhash_get (options, 
+							PORTCFG_PROTO))))
     {
-      report_error ("Port `%s' requires a `" PORTCFG_PROTO "' string field",
-		    portname);
+      guile_error ("Port `%s' requires a `" PORTCFG_PROTO "' string field",
+		   portname);
       FAIL ();
     }
 
@@ -1088,8 +1215,8 @@ guile_define_port (SCM name, SCM args)
 				     &type, txt);
       if (type & ~0xff)
 	{
-	  report_error ("ICMP type `%s' requires a byte (0..255) %s", 
-			PORTCFG_TYPE, txt);
+	  guile_error ("ICMP type `%s' requires a byte (0..255) %s", 
+		       PORTCFG_TYPE, txt);
 	  err = -1;
 	}
       cfg->icmp_type = (unsigned char) (type & 0xff);
@@ -1114,7 +1241,7 @@ guile_define_port (SCM name, SCM args)
 
       /* Check if it is a plain string. */ 
       p = optionhash_get (options, PORTCFG_RECV);
-      if ((str = guile2str (p)) != NULL)
+      if ((str = guile_to_string (p)) != NULL)
 	{
 	  cfg->pipe_recv.name = svz_strdup (str);
 	  cfg->pipe_recv.gid = (unsigned int) -1;
@@ -1123,13 +1250,13 @@ guile_define_port (SCM name, SCM args)
 	  scm_must_free (str);
 	}
       /* Create local optionhash for receiving pipe direction. */
-      else if (p == SCM_UNSPECIFIED)
+      else if (gh_eq_p (p, SCM_UNSPECIFIED))
 	{
-	  report_error ("%s: You have to define a pipe called `%s'",
-			portname, PORTCFG_RECV);
+	  guile_error ("%s: You have to define a pipe called `%s'",
+		       portname, PORTCFG_RECV);
 	  err = -1;
 	}
-      else if ((poptions = guile2optionhash (p, txt, 0)) == NULL)
+      else if ((poptions = guile_to_optionhash (p, txt, 0)) == NULL)
 	{
 	  err = -1; /* Message already emitted. */
 	}
@@ -1145,7 +1272,7 @@ guile_define_port (SCM name, SCM args)
 
       /* Check plain string. */
       p = optionhash_get (options, PORTCFG_SEND);
-      if ((str = guile2str (p)) != NULL)
+      if ((str = guile_to_string (p)) != NULL)
 	{
 	  cfg->pipe_send.name = svz_strdup (str);
 	  cfg->pipe_send.gid = (unsigned int) -1;
@@ -1153,13 +1280,13 @@ guile_define_port (SCM name, SCM args)
 	  cfg->pipe_send.perm = (unsigned int) -1;
 	  scm_must_free (str);
 	}
-      else if (p == SCM_UNSPECIFIED)
+      else if (gh_eq_p (p, SCM_UNSPECIFIED))
 	{
-	  report_error ("%s: You have to define a pipe called `%s'",
-			portname, PORTCFG_SEND);
+	  guile_error ("%s: You have to define a pipe called `%s'",
+		       portname, PORTCFG_SEND);
 	  err = -1;
 	}
-      else if ((poptions = guile2optionhash (p, txt, 0)) == NULL)
+      else if ((poptions = guile_to_optionhash (p, txt, 0)) == NULL)
 	{
 	  err = -1; /* Message already emitted. */
 	}
@@ -1172,8 +1299,8 @@ guile_define_port (SCM name, SCM args)
     }
   else
     {
-      report_error ("Invalid `" PORTCFG_PROTO "' field `%s' in port `%s'",
-		    proto, portname);
+      guile_error ("Invalid `" PORTCFG_PROTO "' field `%s' in port `%s'",
+		   proto, portname);
       FAIL ();
     }
   scm_must_free (proto);
@@ -1192,10 +1319,17 @@ guile_define_port (SCM name, SCM args)
   /* Obtain the access lists "allow" and "deny". */
   if (!(cfg->proto & PROTO_PIPE))
     {
-      cfg->deny = guile2strarray (optionhash_get (options, PORTCFG_DENY),
-				  FUNC_NAME);
-      cfg->allow = guile2strarray (optionhash_get (options, PORTCFG_ALLOW),
-				   FUNC_NAME);
+      SCM list;
+      list = optionhash_get (options, PORTCFG_DENY);
+      if (!gh_eq_p (list, SCM_UNSPECIFIED))
+	cfg->deny = guile_to_strarray (list, FUNC_NAME);
+      else
+	cfg->deny = NULL;
+      list = optionhash_get (options, PORTCFG_ALLOW);
+      if (!gh_eq_p (list, SCM_UNSPECIFIED))
+	cfg->allow = guile_to_strarray (list, FUNC_NAME);
+      else
+	cfg->allow = NULL;
     }
 
   svz_free (txt);
@@ -1209,12 +1343,12 @@ guile_define_port (SCM name, SCM args)
   err |= svz_portcfg_mkaddr (cfg);
 
   if (err)
-    FAIL();
+    FAIL ();
 
   if ((prev = svz_portcfg_add (portname, cfg)) != cfg)
     {
       /* We've overwritten something. Report and dispose. */
-      report_error ("Duplicate definition of port `%s'", portname);
+      guile_error ("Duplicate definition of port `%s'", portname);
       err = -1;
       svz_portcfg_destroy (prev);
     }
@@ -1237,8 +1371,8 @@ guile_define_port (SCM name, SCM args)
 SCM
 guile_bind_server (SCM port, SCM server)
 {
-  char *portname = guile2str (port);
-  char *servername = guile2str (server);
+  char *portname = guile_to_string (port);
+  char *servername = guile_to_string (server);
   svz_server_t *s;
   svz_portcfg_t *p;
   int err = 0;
@@ -1248,28 +1382,26 @@ guile_bind_server (SCM port, SCM server)
   /* Check arguments. */
   if (portname == NULL)
     {
-      report_error ("%s: First argument must be string or symbol",
-		    FUNC_NAME);
+      guile_error ("%s: First argument must be string or symbol", FUNC_NAME);
       FAIL ();
     }
   if (servername == NULL)
     {
-      report_error ("%s: Second argument must be string or symbol", 
-		    FUNC_NAME);
+      guile_error ("%s: Second argument must be string or symbol", FUNC_NAME);
       FAIL ();
     }
 
   /* Check id there is such a port configuration. */
   if ((p = svz_portcfg_get (portname)) == NULL)
     {
-      report_error ("%s: No such port: %s", FUNC_NAME, portname);
+      guile_error ("%s: No such port: %s", FUNC_NAME, portname);
       err++;
     }
 
   /* Get one of the servers in the list. */
   if ((s = svz_server_get (servername)) == NULL)
     {
-      report_error ("%s: No such server: %s", FUNC_NAME, servername);
+      guile_error ("%s: No such server: %s", FUNC_NAME, servername);
       err++;
     }
   
@@ -1291,8 +1423,8 @@ guile_bind_server (SCM port, SCM server)
 /*
  * Converts the given array of strings @var{array} into a guile list.
  */
-static SCM
-strarray2guile (svz_array_t *array)
+SCM
+guile_strarray_to_guile (svz_array_t *array)
 {
   SCM list;
   unsigned long i;
@@ -1307,6 +1439,44 @@ strarray2guile (svz_array_t *array)
   return gh_reverse (list);
 }
 
+/*
+ * Converts the given array of integers @var{array} into a guile list.
+ */
+SCM
+guile_intarray_to_guile (svz_array_t *array)
+{
+  SCM list;
+  unsigned long i;
+  
+  /* Check validity of the give string array. */
+  if (array == NULL)
+    return SCM_UNDEFINED;
+
+  /* Go through all the strings and add these to a guile list. */
+  for (list = SCM_EOL, i = 0; i < svz_array_size (array); i++)
+    list = gh_cons (gh_long2scm ((long) svz_array_get (array, i)), list);
+  return gh_reverse (list);
+}
+
+/*
+ * Converts the given string hash @var{hash} into a guile alist.
+ */
+SCM
+guile_hash_to_guile (svz_hash_t *hash)
+{
+  SCM alist = SCM_EOL, pair;
+  char **key;
+  int n;
+  
+  svz_hash_foreach_key (hash, key, n)
+    {
+      pair = gh_cons (gh_str02scm (key[n]),
+		      gh_str02scm ((char *) svz_hash_get (hash, key[n])));
+      alist = gh_cons (pair, alist);
+    }
+  return alist;
+}
+
 /*
  * Make the list of local interfaces accessable for guile. Returns the
  * local interfaces as a list of ip addresses in dotted decimal form. If
@@ -1331,7 +1501,7 @@ guile_access_interfaces (SCM args)
   array = svz_array_create (0);
   svz_interface_foreach (ifc, n)
     svz_array_add (array, svz_strdup (svz_inet_ntoa (ifc->ipaddr)));
-  list = strarray2guile (array);
+  list = guile_strarray_to_guile (array);
   if (array)
     {
       svz_array_foreach (array, str, n)
@@ -1340,17 +1510,17 @@ guile_access_interfaces (SCM args)
     }
 
   /* Is there an argument given to the guile function ? */
-  if (args != SCM_UNDEFINED)
+  if (!gh_eq_p (args, SCM_UNDEFINED))
     {
       svz_interface_free ();
-      if ((array = guile2strarray (args, FUNC_NAME)) != NULL)
+      if ((array = guile_to_strarray (args, FUNC_NAME)) != NULL)
 	{
 	  svz_array_foreach (array, str, n)
 	    {
 	      if (svz_inet_aton (str, &addr) == -1)
 		{
-		  report_error ("%s: IP address in dotted decimals expected",
-				FUNC_NAME);
+		  guile_error ("%s: IP address in dotted decimals expected",
+			       FUNC_NAME);
 		  guile_global_error = -1;
 		  continue;
 		}
@@ -1385,7 +1555,7 @@ guile_access_loadpath (SCM args)
   GUILE_PRECALL ();
 
   /* Create a guile list containing each search path. */
-  list = strarray2guile (paths);
+  list = guile_strarray_to_guile (paths);
   if (paths)
     {
       svz_array_foreach (paths, str, n)
@@ -1394,9 +1564,9 @@ guile_access_loadpath (SCM args)
     }
   
   /* Set the load path if argument is given. */
-  if (args != SCM_UNDEFINED)
+  if (!gh_eq_p (args, SCM_UNDEFINED))
     {
-      if ((paths = guile2strarray (args, FUNC_NAME)) != NULL)
+      if ((paths = guile_to_strarray (args, FUNC_NAME)) != NULL)
 	svz_dynload_path_set (paths);
     }
   return list;
@@ -1412,7 +1582,7 @@ guile_access_loadpath (SCM args)
 SCM cfunc (SCM arg) {                          \
   char *str;                                   \
   GUILE_PRECALL ();                            \
-  if ((str = guile2str (arg)) == NULL)         \
+  if ((str = guile_to_string (arg)) == NULL)   \
     return SCM_BOOL_F;                         \
   if (!(expression)) {                         \
     scm_must_free (str);                       \
@@ -1438,9 +1608,9 @@ MAKE_STRING_CHECKER (guile_check_server, svz_server_get (str) != NULL)
 static SCM cfunc (SCM args) {                                \
   SCM value = gh_int2scm (cvar); int n;                      \
   GUILE_PRECALL ();                                          \
-  if (args != SCM_UNDEFINED) {                               \
-    if (guile2int (args, &n)) {                              \
-      report_error ("%s: Invalid integer value", FUNC_NAME); \
+  if (!gh_eq_p (args, SCM_UNDEFINED)) {                      \
+    if (guile_to_integer (args, &n)) {                       \
+      guile_error ("%s: Invalid integer value", FUNC_NAME);  \
       guile_global_error = -1;                               \
     } else { cvar = n; } }                                   \
   return value;                                              \
@@ -1453,9 +1623,9 @@ static SCM cfunc (SCM args) {                                \
 static SCM cfunc (SCM args) {                                \
   SCM value = gh_str02scm (cvar); char *str;                 \
   GUILE_PRECALL ();                                          \
-  if (args != SCM_UNDEFINED) {                               \
-    if (NULL == (str = guile2str (args))) {                  \
-      report_error ("%s: Invalid string value", FUNC_NAME);  \
+  if (!gh_eq_p (args, SCM_UNDEFINED)) {                      \
+    if (NULL == (str = guile_to_string (args))) {            \
+      guile_error ("%s: Invalid string value", FUNC_NAME);   \
       guile_global_error = -1;                               \
     } else {                                                 \
       svz_free (cvar);                                       \
@@ -1516,19 +1686,19 @@ guile_init (void)
 }
 
 /*
- * Exception handler for guile. It is called if the evaluation of the given
- * file failed.
+ * Exception handler for guile. It is called if the evaluation of the file
+ * evaluator or a guile procedure call failed.
  */
-static SCM 
+static SCM
 guile_exception (void *data, SCM tag, SCM args)
 {
   /* FIXME: current-load-port is not defined in this state. Why ? */
-  char *str = guile2str (tag);
-  report_error ("Exception due to `%s'", str);
+  char *str = guile_to_string (tag);
+  guile_error ("Exception due to `%s'", str);
   scm_must_free (str);
 
   /* `tag' contains internal exception name */
-  scm_puts ("guile-error: " , scm_current_error_port ());
+  scm_puts ("guile-error: ", scm_current_error_port ());
 
   /* on quit/exit */
   if (gh_null_p (args))
@@ -1538,15 +1708,14 @@ guile_exception (void *data, SCM tag, SCM args)
       return SCM_BOOL_F;
     }
 
-  if (SCM_BOOL_F != gh_car (args))
+  if (!gh_eq_p (gh_car (args), SCM_BOOL_F))
     {
       scm_display (gh_car (args), scm_current_error_port ());
       scm_puts (": ", scm_current_error_port ());
     }
-  scm_simple_format (scm_current_error_port (),
-		     gh_car (gh_cdr (args)),
-		     gh_car (gh_cdr (gh_cdr (args))));
-  scm_puts ("\n", scm_current_error_port ());
+  scm_display_error_message (gh_car (gh_cdr (args)), 
+			     gh_car (gh_cdr (gh_cdr (args))), 
+			     scm_current_error_port ());
   return SCM_BOOL_F;
 }
 
@@ -1560,7 +1729,7 @@ guile_load_config (char *cfgfile)
   guile_global_error = 0;
   guile_init ();
 
-  if (gh_eval_file_with_catch (cfgfile, guile_exception) == SCM_BOOL_F)
+  if (gh_eq_p (gh_eval_file_with_catch (cfgfile, guile_exception), SCM_BOOL_F))
     guile_global_error = -1; 
 
   return guile_global_error ? -1 : 0;
