@@ -1,7 +1,7 @@
 /*
  * irc-proto.c - basic IRC protocol functions
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: irc-proto.c,v 1.22 2000/12/30 01:59:33 ela Exp $
+ * $Id: irc-proto.c,v 1.35 2001/07/06 16:40:02 ela Exp $
  *
  */
 
@@ -39,46 +39,26 @@
 #endif
 
 #ifdef __MINGW32__
-# include <winsock.h>
+# include <winsock2.h>
 #endif
 
 #ifndef __MINGW32__
+# include <sys/types.h>
 # include <netinet/in.h>
 #endif
 
-#include "snprintf.h"
-#include "socket.h"
-#include "alloc.h"
-#include "util.h"
-#include "server.h"
-#include "server-core.h"
-#include "server-socket.h"
-#include "serveez.h"
+#include "libserveez.h"
 #include "irc-core/irc-core.h"
 #include "irc-proto.h"
 #include "irc-event.h"
 #include "irc-server.h"
 #include "irc-config.h"
 
-/*
- * The IRC port configuration.
- */
-portcfg_t irc_port =
-{
-  PROTO_TCP,  /* tcp protocol only */
-  42424,      /* preferred tcp port */
-  "*",        /* preferred local ip address */
-  NULL,       /* calculated automatically */
-  NULL,       /* no receiving pipe */
-  NULL        /* no sending (listening) pipe */
-};
-
 /*
  * The IRC server instance default configuration,
  */
 irc_config_t irc_config =
 {
-  &irc_port,              /* port configuration */
   0,                      /* logged in operators */
   0,                      /* logged in users */
   0,                      /* unknown connections */
@@ -121,33 +101,33 @@ irc_config_t irc_config =
 };
 
 /*
- * Definition of the configuration items processed by sizzle.
+ * Definition of the configuration items.
  */
-key_value_pair_t irc_config_prototype[] =
+svz_key_value_pair_t irc_config_prototype[] =
 {
-  REGISTER_PORTCFG ("port", irc_config.netport, DEFAULTABLE),
-  REGISTER_STR ("MOTD-file", irc_config.MOTD_file, DEFAULTABLE),
-  REGISTER_STR ("INFO-file", irc_config.info_file, DEFAULTABLE),
+  SVZ_REGISTER_STR ("MOTD-file", irc_config.MOTD_file, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STR ("INFO-file", irc_config.info_file, SVZ_ITEM_DEFAULTABLE),
 #if ENABLE_TIMESTAMP
-  REGISTER_INT ("tsdelta", irc_config.tsdelta, DEFAULTABLE),
+  SVZ_REGISTER_INT ("tsdelta", irc_config.tsdelta, SVZ_ITEM_DEFAULTABLE),
 #endif
-  REGISTER_STR ("admininfo", irc_config.admininfo, NOTDEFAULTABLE),
-  REGISTER_STR ("M-line", irc_config.MLine, NOTDEFAULTABLE),
-  REGISTER_STR ("A-line", irc_config.ALine, DEFAULTABLE),
-  REGISTER_STRARRAY ("Y-lines", irc_config.YLine, DEFAULTABLE),
-  REGISTER_STRARRAY ("I-lines", irc_config.ILine, DEFAULTABLE),
-  REGISTER_STRARRAY ("O-lines", irc_config.OLine, DEFAULTABLE),
-  REGISTER_STRARRAY ("o-lines", irc_config.oLine, DEFAULTABLE),
-  REGISTER_STRARRAY ("C-lines", irc_config.CLine, DEFAULTABLE),
-  REGISTER_STRARRAY ("N-lines", irc_config.NLine, DEFAULTABLE),
-  REGISTER_STRARRAY ("K-lines", irc_config.KLine, DEFAULTABLE),
-  REGISTER_END ()
+  SVZ_REGISTER_STR ("admininfo", irc_config.admininfo, 
+		    SVZ_ITEM_NOTDEFAULTABLE),
+  SVZ_REGISTER_STR ("M-line", irc_config.MLine, SVZ_ITEM_NOTDEFAULTABLE),
+  SVZ_REGISTER_STR ("A-line", irc_config.ALine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("Y-lines", irc_config.YLine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("I-lines", irc_config.ILine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("O-lines", irc_config.OLine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("o-lines", irc_config.oLine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("C-lines", irc_config.CLine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("N-lines", irc_config.NLine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_STRARRAY ("K-lines", irc_config.KLine, SVZ_ITEM_DEFAULTABLE),
+  SVZ_REGISTER_END ()
 };
 
 /*
  * Definition of the IRC server.
  */
-server_definition_t irc_server_definition =
+svz_servertype_t irc_server_definition =
 {
   "irc server",        /* long description */
   "irc",               /* short description for instantiating */
@@ -166,11 +146,15 @@ server_definition_t irc_server_definition =
   irc_config_prototype /* configuration prototypes */
 };
 
+/* Static forward declarations. */
+static int irc_delete_channel (irc_config_t *cfg, irc_channel_t *);
+static irc_channel_t *irc_add_channel (irc_config_t *cfg, char *channel);
+
 /*
  * Global IRC server initializer.
  */
 int
-irc_global_init (void)
+irc_global_init (svz_servertype_t *server)
 {
 #if 0
   printf ("sizeof (socket_t)             = %d\n", sizeof (socket_data_t));
@@ -194,7 +178,7 @@ irc_global_init (void)
  * Global IRC server finalizer.
  */
 int
-irc_global_finalize (void)
+irc_global_finalize (svz_servertype_t *server)
 {
   return 0;
 }
@@ -203,44 +187,46 @@ irc_global_finalize (void)
  * Initialization of the IRC server instance.
  */
 int
-irc_init (server_t *server)
+irc_init (svz_server_t *server)
 {
   irc_config_t *cfg = server->cfg;
-  char tmp[256][3];
+  char tmp[3][256];
 
   /* scan the M line (server configuration) */
   if (!cfg->MLine || 
       irc_parse_line (cfg->MLine, "M:%s:%s:%s:%d", 
 		      tmp[0], tmp[1], tmp[2], &cfg->port) != 4)
     {
-      log_printf (LOG_ERROR, "irc: invalid M line: %s\n", 
-		  cfg->MLine ? cfg->MLine : "(nil)");
+      svz_log (LOG_ERROR, "irc: invalid M line: %s\n", 
+	       cfg->MLine ? cfg->MLine : "(nil)");
       return -1;
     }
+  /* FIXME: ???
   if (cfg->port != cfg->netport->port)
     {
-      log_printf (LOG_WARNING, "irc: port in M line clashes\n");
-      cfg->netport->port = cfg->port;
+      svz_log (LOG_WARNING, "irc: port in M line clashes\n");
+      cfg->netport->port = (short) cfg->port;
     }
-  cfg->host = xstrdup (tmp[0]);
-  cfg->realhost = xstrdup (tmp[1]);
-  cfg->info = xstrdup (tmp[2]);
+  */
+  cfg->host = svz_strdup (tmp[0]);
+  cfg->realhost = svz_strdup (tmp[1]);
+  cfg->info = svz_strdup (tmp[2]);
 
   /* scan the A line (administrative information) */
   if (!cfg->ALine ||
       irc_parse_line (cfg->ALine, "A:%s:%s:%s", tmp[0], tmp[1], tmp[2]) != 3)
     {
-      log_printf (LOG_ERROR, "irc: invalid A line: %s\n", 
-		  cfg->ALine ? cfg->ALine : "(nil)");
+      svz_log (LOG_ERROR, "irc: invalid A line: %s\n", 
+	       cfg->ALine ? cfg->ALine : "(nil)");
       return -1;
     }
-  cfg->location1 = xstrdup (tmp[0]);
-  cfg->location2 = xstrdup (tmp[1]);
-  cfg->email = xstrdup (tmp[2]);
+  cfg->location1 = svz_strdup (tmp[0]);
+  cfg->location2 = svz_strdup (tmp[1]);
+  cfg->email = svz_strdup (tmp[2]);
 
   /* initialize hashes and lists */
-  cfg->clients = hash_create (4);
-  cfg->channels = hash_create (4);
+  cfg->clients = svz_hash_create (4);
+  cfg->channels = svz_hash_create (4);
   cfg->clients->equals = irc_string_equal;
   cfg->channels->equals = irc_string_equal;
   cfg->servers = NULL;
@@ -249,8 +235,6 @@ irc_init (server_t *server)
   irc_parse_config_lines (cfg);
   irc_connect_servers (cfg);
 
-  server_bind (server, cfg->netport);
-
   return 0;
 }
 
@@ -258,22 +242,22 @@ irc_init (server_t *server)
  * IRC server instance finalizer.
  */
 int
-irc_finalize (server_t *server)
+irc_finalize (svz_server_t *server)
 {
   irc_config_t *cfg = server->cfg;
   int n;
 
   /* free the MOTD */
   for (n = 0; n < cfg->MOTDs; n++)
-    xfree (cfg->MOTD[n]);
+    svz_free (cfg->MOTD[n]);
 
   /* free configuration hash variables */
-  xfree (cfg->host);
-  xfree (cfg->info);
-  xfree (cfg->realhost);
-  xfree (cfg->location1);
-  xfree (cfg->location2);
-  xfree (cfg->email);
+  svz_free (cfg->host);
+  svz_free (cfg->info);
+  svz_free (cfg->realhost);
+  svz_free (cfg->location1);
+  svz_free (cfg->location2);
+  svz_free (cfg->email);
 
   irc_free_config_lines (cfg);
 
@@ -283,8 +267,8 @@ irc_finalize (server_t *server)
   /* free the server IRC list */
   irc_delete_servers (cfg);
 
-  hash_destroy (cfg->clients);
-  hash_destroy (cfg->channels);
+  svz_hash_destroy (cfg->clients);
+  svz_hash_destroy (cfg->channels);
 
   return 0;
 }
@@ -296,7 +280,7 @@ irc_finalize (server_t *server)
  * ERR_NOTONCHANNEL reply is sent to the socket.
  */
 int
-irc_client_in_channel (socket_t sock,          /* client's socket */
+irc_client_in_channel (svz_socket_t *sock,     /* client's socket */
 		       irc_client_t *client,   /* the client structure */
 		       irc_channel_t *channel) /* the channel to search */
 {
@@ -348,20 +332,22 @@ irc_join_channel (irc_config_t *cfg, irc_client_t *client, char *chan)
 	    }
 	  else
 	    {
-	      channel->client = xrealloc (channel->client,
-					  sizeof (irc_client_t *) * (n + 1));
-	      channel->cflag = xrealloc (channel->cflag,
-					 sizeof (int) * (n + 1));
+	      channel->client = svz_realloc (channel->client,
+					     sizeof (irc_client_t *) * 
+					     (n + 1));
+	      channel->cflag = svz_realloc (channel->cflag,
+					    sizeof (int) * (n + 1));
 	      channel->client[n] = client;
 	      channel->cflag[n] = 0;
 	      channel->clients++;
 #if ENABLE_DEBUG
-	      log_printf (LOG_DEBUG, "irc: %s joined channel %s\n", 
-			  client->nick, channel->name);
+	      svz_log (LOG_DEBUG, "irc: %s joined channel %s\n", 
+		       client->nick, channel->name);
 #endif
 	      n = client->channels;
-	      client->channel = xrealloc (client->channel, 
-					  sizeof (irc_channel_t *) * (n + 1));
+	      client->channel = svz_realloc (client->channel, 
+					     sizeof (irc_channel_t *) * 
+					     (n + 1));
 	      client->channel[n] = channel;
 	      client->channels++;
 	    }
@@ -382,21 +368,21 @@ irc_join_channel (irc_config_t *cfg, irc_client_t *client, char *chan)
 
       /* create one and set the first client as operator */
       channel = irc_add_channel (cfg, chan);
-      channel->client = xmalloc (sizeof (irc_client_t *));
-      channel->cflag = xmalloc (sizeof (int));
+      channel->client = svz_malloc (sizeof (irc_client_t *));
+      channel->cflag = svz_malloc (sizeof (int));
       channel->client[0] = client;
       channel->cflag[0] = MODE_OPERATOR;
       channel->clients = 1;
-      channel->by = xstrdup (client->nick);
+      channel->by = svz_strdup (client->nick);
       channel->since = time (NULL);
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "irc: channel %s created\n", channel->name);
-      log_printf (LOG_DEBUG, "irc: %s joined channel %s\n", 
-		  client->nick, channel->name);
+      svz_log (LOG_DEBUG, "irc: channel %s created\n", channel->name);
+      svz_log (LOG_DEBUG, "irc: %s joined channel %s\n", 
+	       client->nick, channel->name);
 #endif
       n = client->channels;
-      client->channel = xrealloc (client->channel, 
-				  sizeof (irc_channel_t *) * (n + 1));
+      client->channel = svz_realloc (client->channel, 
+				     sizeof (irc_channel_t *) * (n + 1));
       client->channel[n] = channel;
       client->channels++;
     }
@@ -423,20 +409,21 @@ irc_leave_channel (irc_config_t *cfg,
 	  {
 	    channel->client[n] = channel->client[last];
 	    channel->cflag[n] = channel->cflag[last];
-	    channel->client = xrealloc (channel->client, 
-					sizeof (irc_client_t *) * last);
-	    channel->cflag = xrealloc (channel->cflag, sizeof (int) * last);
+	    channel->client = svz_realloc (channel->client, 
+					   sizeof (irc_client_t *) * last);
+	    channel->cflag = svz_realloc (channel->cflag, 
+					  sizeof (int) * last);
 	  }
 	else
 	  {
-	    xfree (channel->client);
+	    svz_free (channel->client);
 	    channel->client = NULL;
-	    xfree (channel->cflag);
+	    svz_free (channel->cflag);
 	    channel->cflag = NULL;
 	  }
 #if ENABLE_DEBUG
-	log_printf (LOG_DEBUG, "irc: %s left channel %s\n",
-		    client->nick, channel->name);
+	svz_log (LOG_DEBUG, "irc: %s left channel %s\n",
+		 client->nick, channel->name);
 #endif
 	/* clear this channel of client's list */
 	last = client->channels - 1;
@@ -446,13 +433,13 @@ irc_leave_channel (irc_config_t *cfg,
 	      if (--client->channels != 0)
 		{
 		  client->channel[i] = client->channel[last];
-		  client->channel = xrealloc (client->channel, 
-					      sizeof (irc_channel_t *) * 
-					      client->channels);
+		  client->channel = svz_realloc (client->channel, 
+						 sizeof (irc_channel_t *) * 
+						 client->channels);
 		}
 	      else
 		{
-		  xfree (client->channel);
+		  svz_free (client->channel);
 		  client->channel = NULL;
 		}
 	      break;
@@ -463,7 +450,7 @@ irc_leave_channel (irc_config_t *cfg,
   if (channel->clients == 0)
     {
 #if ENABLE_DEBUG
-      log_printf (LOG_DEBUG, "irc: channel %s destroyed\n", channel->name);
+      svz_log (LOG_DEBUG, "irc: channel %s destroyed\n", channel->name);
 #endif
       irc_delete_channel (cfg, channel);
     }
@@ -476,7 +463,7 @@ irc_leave_channel (irc_config_t *cfg,
  * Return non zero if there are less than necessary.
  */
 int
-irc_check_args (socket_t sock,           /* the client's socket */
+irc_check_args (svz_socket_t *sock,      /* the client's socket */
 		irc_client_t *client,    /* the irc client itself */
 		irc_config_t *conf,      /* config hash */
 		irc_request_t *request,  /* the request */
@@ -502,7 +489,7 @@ irc_client_absent (irc_client_t *client,  /* requested client */
 		   irc_client_t *rclient) /* who want to know about */
 {
   irc_config_t *cfg;
-  socket_t sock;
+  svz_socket_t *sock;
 
   if (client->flag & UMODE_AWAY)
     {
@@ -526,8 +513,7 @@ irc_leave_all_channels (irc_config_t *cfg,
 {
   irc_channel_t *channel;
   irc_client_t *cl;
-  socket_t sock;
-  socket_t xsock;
+  svz_socket_t *sock, *xsock;
   int i;
 
   sock = client->sock;
@@ -570,7 +556,7 @@ irc_leave_all_channels (irc_config_t *cfg,
  * an IRC client gets lost.
  */
 int
-irc_disconnect (socket_t sock)
+irc_disconnect (svz_socket_t *sock)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *client = sock->data;
@@ -588,7 +574,7 @@ irc_disconnect (socket_t sock)
  * send a PING to a IRC client.
  */
 int
-irc_idle (socket_t sock)
+irc_idle (svz_socket_t *sock)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *client = sock->data;
@@ -667,7 +653,7 @@ irc_callback_t irc_callback[] =
 };
 
 int
-irc_handle_request (socket_t sock, char *request, int len)
+irc_handle_request (svz_socket_t *sock, char *request, int len)
 {
   irc_config_t *cfg = sock->cfg;
   irc_client_t *client = sock->data;
@@ -685,7 +671,7 @@ irc_handle_request (socket_t sock, char *request, int len)
 
   for (n = 0; irc_callback[n].request; n++)
     {
-      if (!util_strcasecmp (irc_callback[n].request, irc_request.request))
+      if (!svz_strcasecmp (irc_callback[n].request, irc_request.request))
 	{
 	  irc_callback[n].count++;
 	  client->recv_bytes += len;
@@ -710,28 +696,28 @@ irc_delete_channel (irc_config_t *cfg, irc_channel_t *channel)
 {
   int n;
 
-  if (hash_contains (cfg->channels, channel))
+  if (svz_hash_contains (cfg->channels, channel))
     {
-      /* xfree() all the channel ban entries */
+      /* svz_free() all the channel ban entries */
       for (n = 0; n < channel->bans; n++)
 	irc_destroy_ban (channel->ban[n]);
       if (channel->ban)
-	xfree (channel->ban);
+	svz_free (channel->ban);
 
-      hash_delete (cfg->channels, channel->name);
+      svz_hash_delete (cfg->channels, channel->name);
       
       if (channel->topic)
-	xfree (channel->topic);
+	svz_free (channel->topic);
       if (channel->topic_by)
-	xfree (channel->topic_by);
+	svz_free (channel->topic_by);
       if (channel->key)
-	xfree (channel->key);
+	svz_free (channel->key);
       if (channel->invite)
-	xfree (channel->invite);
+	svz_free (channel->invite);
 
-      xfree (channel->by);
-      xfree (channel->name);
-      xfree (channel);
+      svz_free (channel->by);
+      svz_free (channel->name);
+      svz_free (channel);
       return 0;
     }
   return -1;
@@ -746,13 +732,13 @@ irc_find_channel (irc_config_t *cfg, char *channel)
 {
   irc_channel_t *chan;
 
-  chan = hash_get (cfg->channels, channel);
+  chan = svz_hash_get (cfg->channels, channel);
   return chan;
 }
 
 /*
  * Find all matching channels in the current channel list. Return NULL if
- * no channel has not been found. You MUST xfree() this list if non-NULL.
+ * no channel has not been found. You MUST svz_free() this list if non-NULL.
  * The delivered array is NULL terminated.
  */
 irc_channel_t **
@@ -761,10 +747,10 @@ irc_regex_channel (irc_config_t *cfg, char *regex)
   irc_channel_t **channel, **fchannel;
   int n, found, size;
 
-  if ((channel = (irc_channel_t **) hash_values (cfg->channels)) != NULL)
+  if ((channel = (irc_channel_t **) svz_hash_values (cfg->channels)) != NULL)
     {
-      size = hash_size (cfg->channels);
-      fchannel = xmalloc (sizeof (irc_channel_t *) * (size + 1));
+      size = svz_hash_size (cfg->channels);
+      fchannel = svz_malloc (sizeof (irc_channel_t *) * (size + 1));
       for (found = n = 0; n < size; n++)
 	{
 	  if (irc_string_regex (channel[n]->name, regex))
@@ -772,17 +758,17 @@ irc_regex_channel (irc_config_t *cfg, char *regex)
 	      fchannel[found++] = channel[n];
 	    }
 	}
-      hash_xfree (channel);
+      svz_hash_xfree (channel);
 
       /* return NULL if there is not channel */
       if (!found)
 	{
-	  xfree (fchannel);
+	  svz_free (fchannel);
 	  return NULL;
 	}
 
       fchannel[found++] = NULL;
-      fchannel = xrealloc (fchannel, sizeof (irc_channel_t *) * found);
+      fchannel = svz_realloc (fchannel, sizeof (irc_channel_t *) * found);
       return fchannel;
     }
   return NULL;
@@ -799,10 +785,10 @@ irc_add_channel (irc_config_t *cfg, char *name)
   if (irc_find_channel (cfg, name))
     return NULL;
 
-  channel = xmalloc (sizeof (irc_channel_t));
+  channel = svz_malloc (sizeof (irc_channel_t));
   memset (channel, 0, sizeof (irc_channel_t));
-  channel->name = xstrdup (name);
-  hash_put (cfg->channels, name, channel);
+  channel->name = svz_strdup (name);
+  svz_hash_put (cfg->channels, name, channel);
   return channel;
 }
 
@@ -814,11 +800,11 @@ irc_add_client_history (irc_config_t *cfg, irc_client_t *cl)
 {
   irc_client_history_t *client;
 
-  client = xmalloc (sizeof (irc_client_history_t));
-  client->nick = xstrdup (cl->nick);
-  client->user = xstrdup (cl->user);
-  client->host = xstrdup (cl->host);
-  client->real = xstrdup (cl->real);
+  client = svz_malloc (sizeof (irc_client_history_t));
+  client->nick = svz_strdup (cl->nick);
+  client->user = svz_strdup (cl->user);
+  client->host = svz_strdup (cl->host);
+  client->real = svz_strdup (cl->real);
   client->next = cfg->history;
   cfg->history = client;
 }
@@ -857,11 +843,11 @@ irc_delete_client_history (irc_config_t *cfg)
   for (client = cfg->history; client; client = old)
     {
       old = client->next;
-      xfree (client->nick);
-      xfree (client->user);
-      xfree (client->host);
-      xfree (client->real);
-      xfree (client);
+      svz_free (client->nick);
+      svz_free (client->user);
+      svz_free (client->host);
+      svz_free (client->real);
+      svz_free (client);
     }
   cfg->history = NULL;
 }
@@ -879,27 +865,27 @@ irc_delete_client (irc_config_t *cfg, irc_client_t *client)
     {
       /* put this client into the history list */
       irc_add_client_history (cfg, client);
-      if (hash_delete (cfg->clients, client->nick) == NULL)
+      if (svz_hash_delete (cfg->clients, client->nick) == NULL)
 	ret = -1;
-      xfree (client->nick);
+      svz_free (client->nick);
     }
 
   /* free all client properties */
   if (client->real)
-    xfree (client->real);
+    svz_free (client->real);
   if (client->user)
-    xfree (client->user);
+    svz_free (client->user);
   if (client->host)
-    xfree (client->host);
+    svz_free (client->host);
   if (client->server)
-    xfree (client->server);
+    svz_free (client->server);
   if (client->channel)
-    xfree (client->channel);
+    svz_free (client->channel);
   if (client->pass)
-    xfree (client->pass);
+    svz_free (client->pass);
   if (client->away)
-    xfree (client->away);
-  xfree (client);
+    svz_free (client->away);
+  svz_free (client);
   cfg->users--;
 
   return ret;
@@ -916,19 +902,19 @@ irc_find_userhost (irc_config_t *cfg, char *user, char *host)
   irc_client_t *fclient;
   int n;
 
-  if ((client = (irc_client_t **) hash_values (cfg->clients)) != NULL)
+  if ((client = (irc_client_t **) svz_hash_values (cfg->clients)) != NULL)
     {
-      for (n = 0; n < hash_size (cfg->clients); n++)
+      for (n = 0; n < svz_hash_size (cfg->clients); n++)
 	{
 	  if (!strcmp (client[n]->user, user) && 
 	      !strcmp (client[n]->host, host))
 	    {
 	      fclient = client[n];
-	      xfree (client);
+	      svz_hash_xfree (client);
 	      return fclient;
 	    }
 	}
-      hash_xfree (client);
+      svz_hash_xfree (client);
     }
   return NULL;
 }
@@ -942,7 +928,7 @@ irc_find_nick (irc_config_t *cfg, char *nick)
 {
   irc_client_t *client;
 
-  if ((client = hash_get (cfg->clients, nick)) != NULL)
+  if ((client = svz_hash_get (cfg->clients, nick)) != NULL)
     {
       return client;
     }
@@ -951,7 +937,7 @@ irc_find_nick (irc_config_t *cfg, char *nick)
 
 /*
  * Find all matching nicks in the current client list. Return NULL if
- * no nick has not been found. You MUST xfree() this array if it is
+ * no nick has not been found. You MUST svz_free() this array if it is
  * non-NULL. The delivered clients are NULL terminated.
  */
 irc_client_t **
@@ -960,10 +946,10 @@ irc_regex_nick (irc_config_t *cfg, char *regex)
   irc_client_t **client, **fclient;
   int n, found, size;
 
-  if ((client = (irc_client_t **) hash_values (cfg->clients)) != NULL)
+  if ((client = (irc_client_t **) svz_hash_values (cfg->clients)) != NULL)
     {
-      size = hash_size (cfg->clients);
-      fclient = xmalloc (sizeof (irc_client_t *) * (size + 1));
+      size = svz_hash_size (cfg->clients);
+      fclient = svz_malloc (sizeof (irc_client_t *) * (size + 1));
       for (found = n = 0; n < size; n++)
 	{
 	  if (irc_string_regex (client[n]->nick, regex))
@@ -971,17 +957,17 @@ irc_regex_nick (irc_config_t *cfg, char *regex)
 	      fclient[found++] = client[n];
 	    }
 	}
-      hash_xfree (client);
+      svz_hash_xfree (client);
 
       /* return NULL if there is not client */
       if (!found)
 	{
-	  xfree (fclient);
+	  svz_free (fclient);
 	  return NULL;
 	}
 
       fclient[found++] = NULL;
-      fclient = xrealloc (fclient, sizeof (irc_client_t *) * found);
+      fclient = svz_realloc (fclient, sizeof (irc_client_t *) * found);
       return fclient;
     }
   return NULL;
@@ -996,7 +982,7 @@ irc_add_client (irc_config_t *cfg, irc_client_t *client)
   if (irc_find_nick (cfg, client->nick))
     return NULL;
 
-  hash_put (cfg->clients, client->nick, client);
+  svz_hash_put (cfg->clients, client->nick, client);
   return client;
 }
 
@@ -1009,7 +995,7 @@ irc_create_client (irc_config_t *cfg)
 {
   irc_client_t *client;
 
-  client = xmalloc (sizeof (irc_client_t));
+  client = svz_malloc (sizeof (irc_client_t));
   memset (client, 0, sizeof (irc_client_t));
   cfg->users++;
   return client;
@@ -1019,7 +1005,7 @@ irc_create_client (irc_config_t *cfg)
  * Print a formatted string to the socket SOCK.
  */
 int
-irc_printf (socket_t sock, const char *fmt, ...)
+irc_printf (svz_socket_t *sock, const char *fmt, ...)
 {
   va_list args;
   static char buffer[VSNPRINTF_BUF_SIZE];
@@ -1029,14 +1015,14 @@ irc_printf (socket_t sock, const char *fmt, ...)
     return 0;
 
   va_start (args, fmt);
-  len = vsnprintf (buffer, VSNPRINTF_BUF_SIZE, fmt, args);
+  len = svz_vsnprintf (buffer, VSNPRINTF_BUF_SIZE, fmt, args);
   va_end (args);
 
   /* Just to be sure... */
   if (len > sizeof (buffer))
     len = sizeof (buffer);
 
-  if ((len = sock_write (sock, buffer, len)) != 0)
+  if ((len = svz_sock_write (sock, buffer, len)) != 0)
     {
       sock->flags |= SOCK_FLAG_KILLED;
     }
