@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: coserver.c,v 1.19 2001/07/06 16:40:03 ela Exp $
+ * $Id: coserver.c,v 1.22 2001/10/07 17:10:28 ela Exp $
  *
  */
 
@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <sys/types.h>
 #include <errno.h>
 #if HAVE_UNISTD_H
@@ -133,13 +134,13 @@ svz_coservertype_t svz_coservertypes[] =
      place coserver callbacks and identification here */
 
   { COSERVER_REVERSE_DNS, "reverse dns", 
-    reverse_dns_handle_request, 1, reverse_dns_init },
+    reverse_dns_handle_request, 1, reverse_dns_init, 0 },
 
   { COSERVER_IDENT, "ident", 
-    ident_handle_request, 1, NULL},
+    ident_handle_request, 1, NULL, 0},
 
   { COSERVER_DNS, "dns", 
-    dns_handle_request, 1, NULL }
+    dns_handle_request, 1, NULL, 0 }
 };
 
 /*
@@ -400,41 +401,23 @@ svz_coserver_activate (int type)
 #endif /* ENABLE_DEBUG */
 }
 
-/*
- * Call this routine whenever there is time, e.g. within the timeout of 
- * the @code{select()} statement. Indeed I built it in the 
- * @code{svz_periodic_tasks()} statement. The routine checks if there was 
- * any response from an active coserver.
- */
-void
-svz_coserver_check (void)
-{
-  svz_coserver_t *coserver;
-  svz_socket_t *sock;
-  int n;
-  
-  /* go through all coservers */
-  svz_array_foreach (svz_coservers, coserver, n)
-    {
-      sock = coserver->sock;
-      while (sock->recv_buffer_fill > 0)
-	{
-#if ENABLE_DEBUG
-	  svz_log (LOG_DEBUG, "%s: coserver response detected\n",
-		   svz_coservertypes[coserver->type].name);
-#endif
-	  /* find a full response within the receive buffer */
-	  if (sock->check_request)
-	    {
-	      EnterCriticalSection (&coserver->sync);
-	      sock->check_request (sock);
-	      LeaveCriticalSection (&coserver->sync);
-	    }
-	}
-    }
-}
 #endif /* __MINGW32__ */
 
+/*
+ * Return the number of currently running coservers with the type @var{type}.
+ */
+static int
+svz_coserver_count (int type)
+{
+  int n, count = 0;
+  svz_coserver_t *coserver;
+
+  svz_array_foreach (svz_coservers, coserver, n)
+    if (coserver->type == type)
+      count++;
+  return count;
+}
+
 /*
  * Delete the n'th internal coserver from coserver array.
  */
@@ -649,8 +632,10 @@ svz_coserver_closeall (void)
 	close (sock->file_desc);
       if (sock->flags & SOCK_FLAG_PIPE)
 	{
-	  close (sock->pipe_desc[READ]);
-	  close (sock->pipe_desc[WRITE]);
+	  if (sock->pipe_desc[READ] >= 2)
+	    close (sock->pipe_desc[READ]);
+	  if (sock->pipe_desc[WRITE] >= 2)
+	    close (sock->pipe_desc[WRITE]);
 	}
     }
   svz_file_closeall ();
@@ -910,6 +895,7 @@ svz_coserver_start (int type)
 
 #endif /* __MINGW32__ and Unices */
 
+  svz_coservertypes[coserver->type].last_start = (long) time (NULL);
   sock->data = coserver;
   sock->check_request = svz_coserver_check_request;
   sock->handle_request = svz_coserver_handle_request;
@@ -917,6 +903,65 @@ svz_coserver_start (int type)
   return sock;
 }
 
+/*
+ * Call this routine whenever there is time, e.g. within the timeout of 
+ * the @code{select()} statement. Indeed I built it in the 
+ * @code{svz_periodic_tasks()} statement. Under Wind32 the routine checks 
+ * if there was any response from an active coserver. Moreover it keeps
+ * the coserver threads/processes alive. If one of the coservers dies due
+ * to buffer overrun or might be overloaded this function starts a new one.
+ */
+void
+svz_coserver_check (void)
+{
+  svz_coserver_t *coserver;
+  svz_coservertype_t *ctype;
+  svz_socket_t *sock;
+  int n;
+  
+#ifdef __MINGW32__
+  /* go through all coservers */
+  svz_array_foreach (svz_coservers, coserver, n)
+    {
+      sock = coserver->sock;
+      while (sock->recv_buffer_fill > 0)
+	{
+#if ENABLE_DEBUG
+	  svz_log (LOG_DEBUG, "%s: coserver response detected\n",
+		   svz_coservertypes[coserver->type].name);
+#endif
+	  /* find a full response within the receive buffer */
+	  if (sock->check_request)
+	    {
+	      EnterCriticalSection (&coserver->sync);
+	      sock->check_request (sock);
+	      LeaveCriticalSection (&coserver->sync);
+	    }
+	}
+    }
+#endif /* __MINGW32__ */
+
+  /* check the number of coserver instances of each coserver type */
+  for (n = 0; n < MAX_COSERVER_TYPES; n++)
+    {
+      ctype = &svz_coservertypes[n];
+      if (svz_coserver_count (ctype->type) < ctype->instances &&
+	  ((long) time (NULL)) - ctype->last_start >= 3)
+	svz_coserver_start (ctype->type);
+    }
+
+  /* restart coserver instances if buffer overrun is in sight (send buffer
+     fill >= 75 percent) */
+  svz_array_foreach (svz_coservers, coserver, n)
+    {
+      ctype = &svz_coservertypes[coserver->type];
+      sock = coserver->sock;
+      if (sock->send_buffer_fill * 100 / sock->send_buffer_size >= 75 &&
+	  ((long) time (NULL)) - ctype->last_start >= 3)
+	svz_coserver_start (coserver->type);
+    }
+}
+
 /*
  * Create a single coserver with the given type @var{type}.
  */
@@ -947,9 +992,7 @@ svz_coserver_init (void)
       if (coserver->init)
 	coserver->init ();
       for (i = 0; i < coserver->instances; i++)
-	{
-	  svz_coserver_start (coserver->type);
-	}
+	svz_coserver_start (coserver->type);
     }
 
   return 0;
