@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: icmp-socket.c,v 1.15 2001/07/06 22:49:06 ela Exp $
+ * $Id: icmp-socket.c,v 1.17 2001/09/19 09:49:18 ela Exp $
  *
  */
 
@@ -254,7 +254,7 @@ static char svz_icmp_buffer[IP_HEADER_SIZE + ICMP_HEADER_SIZE + ICMP_MSG_SIZE];
  * Get ICMP header from plain data.
  */
 static svz_icmp_header_t *
-svz_icmp_get_header (byte *data)
+svz_icmp_get_header (svz_uint8_t *data)
 {
   static svz_icmp_header_t hdr;
   unsigned short uint16;
@@ -279,11 +279,11 @@ svz_icmp_get_header (byte *data)
 /*
  * Create ICMP header (data block) from given structure.
  */
-static byte *
+static svz_uint8_t *
 svz_icmp_put_header (svz_icmp_header_t *hdr)
 {
-  static byte buffer[ICMP_HEADER_SIZE];
-  byte *data = buffer;
+  static svz_uint8_t buffer[ICMP_HEADER_SIZE];
+  svz_uint8_t *data = buffer;
   unsigned short uint16;
 
   *data++ = hdr->type;
@@ -312,10 +312,10 @@ svz_icmp_put_header (svz_icmp_header_t *hdr)
  * ICMP_DISCONNECT when we received an disconnection signal.
  */
 static int
-svz_icmp_check_packet (svz_socket_t *sock, byte *data, int len)
+svz_icmp_check_packet (svz_socket_t *sock, svz_uint8_t *data, int len)
 {
   int length;
-  byte *p = data;
+  svz_uint8_t *p = data;
   svz_icmp_header_t *header;
 
   /* First check the IP header. */
@@ -448,7 +448,7 @@ svz_icmp_read_socket (svz_socket_t *sock)
        * Check the ICMP packet and put the packet load only into the
        * receive buffer of the socket structure.
        */
-      trunc = svz_icmp_check_packet (sock, (byte *) svz_icmp_buffer, 
+      trunc = svz_icmp_check_packet (sock, (svz_uint8_t *) svz_icmp_buffer, 
 				     num_read);
       if (trunc >= 0)
 	{
@@ -465,6 +465,10 @@ svz_icmp_read_socket (svz_socket_t *sock)
 		  svz_icmp_buffer + trunc, num_read);
 	  sock->recv_buffer_fill += num_read;
 
+	  /* Check access lists. */
+	  if (svz_sock_check_access (sock, sock) < 0)
+	    return 0;
+
 	  if (sock->check_request)
 	    sock->check_request (sock);
 	}
@@ -564,7 +568,7 @@ svz_icmp_write_socket (svz_socket_t *sock)
  * signaling that this connection is going down soon.
  */
 int
-svz_icmp_send_control (svz_socket_t *sock, byte type)
+svz_icmp_send_control (svz_socket_t *sock, svz_uint8_t type)
 {
   static char *buffer = svz_icmp_buffer;
   svz_icmp_header_t hdr;
@@ -628,7 +632,7 @@ svz_icmp_write (svz_socket_t *sock, char *buf, int length)
       /* Create ICMP header and put it in front of packet load. */
       hdr.type = sock->itype;
       hdr.code = ICMP_SERVEEZ_DATA;
-      hdr.checksum = svz_raw_ip_checksum ((byte *) buf, size);
+      hdr.checksum = svz_raw_ip_checksum ((svz_uint8_t *) buf, size);
       hdr.ident = (unsigned short) (getpid () + sock->id);
       hdr.sequence = sock->send_seq++;
       hdr.port = sock->remote_port;
