@@ -20,7 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: server-core.c,v 1.24 2001/07/05 21:35:26 ela Exp $
+ * $Id: server-core.c,v 1.30 2001/09/29 01:05:02 ela Exp $
  *
  */
 
@@ -31,7 +31,12 @@
 #define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
+
+/* Some Unices define the strsignal() function depending on 
+   this definition. */
+#define __EXTENSIONS__
 #include <string.h>
+
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
@@ -130,12 +135,13 @@ svz_socket_t *svz_sock_root = NULL;
 svz_socket_t *svz_sock_last = NULL;
 
 /*
- * @var{svz_sock_lookup_table} is used to speed up references to socket
- * structures by socket's id.
+ * The @var{svz_sock_lookup_table} array is used to speed up references to 
+ * socket structures by socket's id.
  */
-static svz_socket_t *svz_sock_lookup_table[SOCK_MAX_ID];
+static svz_socket_t **svz_sock_lookup_table = NULL;
 static int svz_sock_id = 0;
 static int svz_sock_version = 0;
+static int svz_sock_limit = 1024; /* Must be binary size ! */
 
 /* Pointer to argv[0]. */
 static char *svz_executable_file = NULL;
@@ -265,12 +271,17 @@ svz_signal_handler (int sig)
 #endif
 }
 
-/* 64 is hopefully a safe bet, kill(1) accepts 0..64, *sigh* */
+/* 65 is hopefully a safe bet, kill(1) accepts 0..64, *sigh* */
 #define SVZ_NUMBER_OF_SIGNALS 65
 
 /* Cached results of strsignal calls. */
 static svz_array_t *svz_signal_strings = NULL;
 
+/* On some platforms strsignal() can be resolved but is nowhere declared. */
+#if defined (HAVE_STRSIGNAL) && !defined (DECLARED_STRSIGNAL)
+extern char * strsignal (int);
+#endif
+
 /*
  * Prepare library so that @code{svz_strsignal()} works. Called
  * from @code{svz_boot()}.
@@ -662,10 +673,18 @@ svz_sock_check_frequency (svz_socket_t *parent, svz_socket_t *child)
 int
 svz_sock_check_access (svz_socket_t *parent, svz_socket_t *child)
 {
-  svz_portcfg_t *port = parent->port;
+  svz_portcfg_t *port;
   char *ip;
   int n, ret;
-  char *remote = svz_inet_ntoa (child->remote_addr);
+  char *remote;
+
+  /* Check arguments and return if this function cannot work. */
+  if (parent == NULL || child == NULL || parent->port == NULL)
+    return 0;
+
+  /* Get port configuration and remote address. */
+  port = parent->port;
+  remote = svz_inet_ntoa (child->remote_addr);
 
   /* Check the deny IP addresses. */
   if (port->deny)
@@ -787,7 +806,7 @@ svz_sock_find (int id, int version)
 {
   svz_socket_t *sock;
 
-  if (id & ~(SOCK_MAX_ID - 1))
+  if (id & ~(svz_sock_limit - 1))
     {
       svz_log (LOG_FATAL, "socket id %d is invalid\n", id);
       return NULL;
@@ -804,6 +823,27 @@ svz_sock_find (int id, int version)
   return svz_sock_lookup_table[id];
 }
 
+/*
+ * Create the socket lookup table initially. Must be called from 
+ * @code{svz_boot()}.
+ */
+void
+svz_sock_table_create (void)
+{
+  svz_sock_lookup_table = svz_calloc (svz_sock_limit * 
+				      sizeof (svz_socket_t *));
+}
+
+/*
+ * Destroy the socket lookup table finally. Must be called from 
+ * @code{svz_halt()}.
+ */
+void
+svz_sock_table_destroy (void)
+{
+  svz_free_and_zero (svz_sock_lookup_table);
+}
+
 /*
  * Calculate unique socket structure id and assign a version for a 
  * given @var{sock}. The version is for validating socket structures. It is 
@@ -814,18 +854,27 @@ svz_sock_unique_id (svz_socket_t *sock)
 {
   int i;
 
-  for (i = 0; i < SOCK_MAX_ID; i++) 
+  for (i = 0; i < svz_sock_limit; i++) 
     {
       svz_sock_id++;
-      svz_sock_id &= (SOCK_MAX_ID - 1);
+      svz_sock_id &= (svz_sock_limit - 1);
 
       if (NULL == svz_sock_lookup_table[svz_sock_id])
 	break;
     }
 
-  /* ensure global limit for now */
-  if (i == SOCK_MAX_ID)
-    abort ();
+  /* ensure global limit, resize the lookup table if necessary */
+  if (i == svz_sock_limit)
+    {
+      svz_sock_lookup_table = svz_realloc (svz_sock_lookup_table,
+					   svz_sock_limit * 2 * 
+					   sizeof (svz_socket_t *));
+      memset (&svz_sock_lookup_table[svz_sock_limit], 0,
+	      svz_sock_limit * sizeof (svz_socket_t *));
+      svz_sock_id = svz_sock_limit;
+      svz_sock_limit *= 2;
+      svz_log (LOG_NOTICE, "lookup table enlarged to %d\n", svz_sock_limit);
+    }
 
   sock->id = svz_sock_id;
   sock->version = svz_sock_version++;
@@ -905,6 +954,15 @@ svz_sock_schedule_for_shutdown (svz_socket_t *sock)
 #endif /* ENABLE_DEBUG */
 
       sock->flags |= SOCK_FLAG_KILLED;
+
+      /* Shutdown each child for listeners. */
+      if (sock->flags & SOCK_FLAG_LISTENING)
+	{
+	  svz_socket_t *child;
+	  svz_sock_foreach (child)
+	    if (svz_sock_getparent (child) == sock)
+	      svz_sock_schedule_for_shutdown (child);
+	}
     }
   return 0;
 }
@@ -947,10 +1005,9 @@ svz_periodic_tasks (void)
       sock = sock->next;
     }
 
-#ifdef __MINGW32__
-  /* check regularly for internal coserver responses...  */
+  /* check regularly for internal coserver responses and keep coservers
+     alive */
   svz_coserver_check ();
-#endif /* not __MINGW32__ */
 
   /* run the server instance timer routines */
   svz_server_notifiers ();
