@@ -20,7 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: util.c,v 1.15 2001/07/02 11:46:34 ela Exp $
+ * $Id: util.c,v 1.23 2001/09/27 15:47:36 ela Exp $
  *
  */
 
@@ -70,6 +70,7 @@
 # include <sys/utsname.h>
 #endif
 
+#include "libserveez/alloc.h"
 #include "libserveez/snprintf.h"
 #include "libserveez/boot.h"
 #include "libserveez/windoze.h"
@@ -188,24 +189,30 @@ svz_hexdump (FILE *out,    /* output FILE stream */
   return 0;
 }
 
+/* On some platforms @code{hstrerror()} can be resolved but is not declared
+   anywhere. That is why we do it here by hand. */
+#if defined (HAVE_HSTRERROR) && !defined (DECLARED_HSTRERROR)
+extern char * hstrerror (int);
+#endif
+
 /*
  * This is the @code{hstrerror()} wrapper function, depending on the 
  * configuration file @file{config.h}.
  */
-const char *
+char *
 svz_hstrerror (void)
 {
 #if HAVE_HSTRERROR
 # if HAVE_H_ERRNO
-  return hstrerror (h_errno);
+  return (char *) hstrerror (h_errno);
 # else
-  return hstrerror (errno);
+  return (char *) hstrerror (errno);
 # endif
 #else /* not HAVE_HSTRERROR */
 # if HAVE_H_ERRNO
-  return strerror (h_errno);
+  return (char *) strerror (h_errno);
 # else
-  return strerror (errno);
+  return (char *) strerror (errno);
 # endif
 #endif /* not HAVE_HSTRERROR */
 }
@@ -278,7 +285,8 @@ svz_tolower (char *str)
 
   while (*p)
     {
-      *p = (char) (isupper ((byte) * p) ? tolower ((byte) * p) : *p);
+      *p = (char) (isupper ((svz_uint8_t) * p) ? 
+		   tolower ((svz_uint8_t) * p) : *p);
       p++;
     }
   return str;
@@ -477,7 +485,6 @@ char *
 svz_syserror (int nr)
 {
   static char message[MESSAGE_BUF_SIZE];
-  LPTSTR error;
 
   /* save the last error */
   svz_errno = nr;
@@ -494,20 +501,17 @@ svz_syserror (int nr)
     nr = errno;
 
   /* return a sys error */
-  if (0 ==
-      FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER |
-		     FORMAT_MESSAGE_FROM_SYSTEM |
-		     FORMAT_MESSAGE_ARGUMENT_ARRAY, NULL, nr,
-		     MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
-		     (char *) &error, 0, NULL))
+  if (0 == FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM |
+			  FORMAT_MESSAGE_ARGUMENT_ARRAY, NULL, nr,
+			  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+			  (char *) message, MESSAGE_BUF_SIZE, NULL))
     {
-      sprintf (message, "FormatMessage: error code %ld", GetLastError ());
+      sprintf (message, "FormatMessage (%d): error code %ld", 
+	       nr, GetLastError ());
       return message;
     }
 
-  strcpy (message, error);
   message[strlen (message) - 2] = 0;
-  LocalFree (error);
   return message;
 }
 
@@ -516,7 +520,7 @@ svz_syserror (int nr)
  * is setup in @code{svz_version()} and can be @code{Win32s} for Windows 3.x,
  * @code{Win95} for Windows 95, @code{Win98} for Windows 98, @code{WinNT3x}
  * for Windows NT 3.x, @code{WinNT4x} for Windows NT 4.x, @code{Win2k} for
- * Windows 2000 and @code{WinME} for Windows ME.
+ * Windows 2000, @code{WinXP} for Windows XP and @code{WinME} for Windows ME.
  */
 int svz_os_version = 0;
 
@@ -535,7 +539,7 @@ svz_sys_version (void)
 
 #ifdef __MINGW32__
   static char ver[][6] =
-    { " 32s", " 95", " 98", " NT", " NT", " 2000", " ME" };
+    { " 32s", " 95", " 98", " NT", " NT", " 2000", " XP", " ME" };
   OSVERSIONINFO osver;
 #elif HAVE_SYS_UTSNAME_H
   struct utsname buf;
@@ -556,13 +560,15 @@ svz_sys_version (void)
     {
       switch (osver.dwPlatformId)
 	{
-	case VER_PLATFORM_WIN32_NT: /* NT or Windows 2000 */
+	case VER_PLATFORM_WIN32_NT: /* NT, Windows 2000 or Windows XP */
 	  if (osver.dwMajorVersion == 4)
 	    svz_os_version = WinNT4x;
 	  else if (osver.dwMajorVersion <= 3)
 	    svz_os_version = WinNT3x;
-	  else if (osver.dwMajorVersion == 5)
+	  else if (osver.dwMajorVersion == 5 && osver.dwMinorVersion < 1)
 	    svz_os_version = Win2k;
+	  else if (osver.dwMajorVersion >= 5)
+	    svz_os_version = WinXP;
 	  break;
 
 	case VER_PLATFORM_WIN32_WINDOWS: /* Win95 or Win98 */
@@ -635,6 +641,28 @@ svz_atoi (char *str)
   return i;
 }
 
+/*
+ * Returns the current working directory. The returned pointer needs to be
+ * @code{svz_free()}'ed after usage.
+ */
+char *
+svz_getcwd (void)
+{
+  char *buf, *dir;
+  int len = 64;
+
+  buf = dir = NULL;
+  do
+    {
+      buf = svz_realloc (buf, len);
+      dir = getcwd (buf, len);
+      len *= 2;
+    }
+  while (dir == NULL);
+
+  return dir;
+}
+
 /*
  * This routine checks for the current and maximum limit of open files
  * of the current process. The function heavily depends on the underlying
