@@ -18,36 +18,32 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: sntp-proto.h,v 1.1 2000/12/15 14:13:18 ela Exp $
+ * $Id: sntp-proto.h,v 1.5 2001/05/19 23:04:58 ela Exp $
  *
  */
 
 #ifndef __SNTP_PROTO_H__
-#define __SNTP_PROTO_H__
+#define __SNTP_PROTO_H__ 1
 
 #if HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#define _GNU_SOURCE
-#include "server.h"
-#include "socket.h"
-
 /*
  * Protocol server specific configuration.
  */
 typedef struct
 {
-  portcfg_t *port;
+  int nothing;
 }
 sntp_config_t;
 
-int sntp_init (server_t *server);
-int sntp_handle_request (socket_t sock, char *packet, int len);
+int sntp_init (svz_server_t *server);
+int sntp_handle_request (svz_socket_t *sock, char *packet, int len);
 
 /*
  * This server's definition.
  */
-extern server_definition_t sntp_server_definition;
+extern svz_servertype_t sntp_server_definition;
 
-#endif /* __SNTP_PROTO_H__ */
+#endif /* not __SNTP_PROTO_H__ */
